/* This file is part of the Springlobby (GPL v2 or later), see COPYING */

#ifndef SPRINGLOBBY_HEADERGUARD_CONFIG_H
#define SPRINGLOBBY_HEADERGUARD_CONFIG_H

#undef VERSION

/* the git tag / commit we build from */
#define VERSION "@SPRINGLOBBY_REV@"

#endif	/* SPRINGLOBBY_HEADERGUARD_CONFIG_H */
