/* This file is part of the Springlobby (GPL v2 or later), see COPYING */

#ifndef SPRINGLOBBY_HEADERGUARD_COUNTRYCODES_H
#define SPRINGLOBBY_HEADERGUARD_COUNTRYCODES_H

class wxString;

wxString GetFlagNameFromCountryCode(const wxString& cc);

#endif // SPRINGLOBBY_HEADERGUARD_COUNTRYCODES_H
