/* This file is part of the Springlobby (GPL v2 or later), see COPYING */

#ifndef SPRINGLOBBY_HEADERGUARD_GLOBALAUIMANAGER_H
#define SPRINGLOBBY_HEADERGUARD_GLOBALAUIMANAGER_H

#include <wx/aui/aui.h>

static wxAuiManager* m_global_aui_mngr = 0;

#endif // SPRINGLOBBY_HEADERGUARD_GLOBALAUIMANAGER_H
