/* This file is part of the Springlobby (GPL v2 or later), see COPYING */

#ifndef SPRINGLOBBY_HEADERGUARD_MIXINS_HH
#define SPRINGLOBBY_HEADERGUARD_MIXINS_HH

#include <boost/noncopyable.hpp>

namespace SL
{
typedef boost::noncopyable NonCopyable;
}

#endif // SPRINGLOBBY_HEADERGUARD_MIXINS_HH
