/* This file is part of the Springlobby (GPL v2 or later), see COPYING */

#ifndef SPRINGLOBBY_VERSION_H
#define SPRINGLOBBY_VERSION_H

#include <string>

const std::string getSpringlobbyVersion();
const std::string getSpringlobbyAgent();
const std::string getSpringlobbyName(bool lowercase = false);

#endif
